package sgc.portal.com.com.persistence;

import sgc.portal.base.util.EasyHashMap;
import sgc.portal.com.com.model.Dtlauth;
import sgc.portal.com.com.model.Stddauth;
import sgc.portal.com.com.model.User;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;


/**
 * 프로그램 목록 매퍼  DAO
 *
 * @author KyungHee Jang(khjang@sgc.co.kr)
 * @version 1.0.0
 * @since  2020. 8. 2.
 */


@Mapper
public interface  ComUserMngMapper {

    List<EasyHashMap<String, Object>> selectComUserMng(User user);

    void insertComUserMng(User user);

    void updateComUserMng(User user);

    void deleteComUserMng(User user);

    List<EasyHashMap<String, Object>> selectComUserStddauthMng(Stddauth stddauth);

    void insertComUserStddauthMng(Stddauth stddauth);

    void updateComUserStddauthMng(Stddauth stddauth);

    void deleteComUserStddauthMng(Stddauth stddauth);

    List<EasyHashMap<String, Object>> selectComUserDtlauthMng(Dtlauth dtlauth);

    void insertComUserDtlauthMng(Dtlauth dtlauth);

    void deleteComUserDtlauthMng(Dtlauth dtlauth);

    List<EasyHashMap<String, Object>> createUsers();
}

